@@ -13,10 +13,10 @@
 
 """
 
-from cssselect.parser import (parse, Selector, SelectorError,
-                              SelectorSyntaxError)
+from cssselect.parser import (parse, Selector, FunctionalPseudoElement,
+                              SelectorError, SelectorSyntaxError)
 from cssselect.xpath import GenericTranslator, HTMLTranslator, ExpressionError
 
 
-VERSION = '0.7'
+VERSION = '0.9'
 __version__ = VERSION
