@@ -23,7 +23,9 @@ import unittest
 from lxml import etree, html
 from cssselect import (parse, GenericTranslator, HTMLTranslator,
                        SelectorSyntaxError, ExpressionError)
-from cssselect.parser import tokenize, parse_series, _unicode
+from cssselect.parser import (tokenize, parse_series, _unicode,
+                              FunctionalPseudoElement)
+from cssselect.xpath import _unicode_safe_getattr, XPathExpr
 
 
 if sys.version_info[0] < 3:
@@ -39,16 +41,15 @@ else:
 class TestCssselect(unittest.TestCase):
     def test_tokenizer(self):
         tokens = [
-            repr(item).replace("u'", "'")  # Py 2/3
-            for item in tokenize(
+            _unicode(item) for item in tokenize(
                 u(r'E\ é > f [a~="y\"x"]:nth(/* fu /]* */-3.7)'))]
         assert tokens == [
-            "<IDENT 'E é' at 0>",
+            u("<IDENT 'E é' at 0>"),
             "<S ' ' at 4>",
             "<DELIM '>' at 5>",
             "<S ' ' at 6>",
             # the no-break space is not whitespace in CSS
-            r"<IDENT 'f\xa0' at 7>",
+            u("<IDENT 'f ' at 7>"),  # f\xa0
             "<DELIM '[' at 9>",
             "<IDENT 'a' at 10>",
             "<DELIM '~' at 11>",
@@ -151,6 +152,7 @@ class TestCssselect(unittest.TestCase):
             result = []
             for selector in parse(css):
                 pseudo = selector.pseudo_element
+                pseudo = _unicode(pseudo) if pseudo else pseudo
                 # No Symbol here
                 assert pseudo is None or type(pseudo) is _unicode
                 selector = repr(selector.parsed_tree).replace("(u'", "('")
@@ -167,17 +169,21 @@ class TestCssselect(unittest.TestCase):
         assert parse_one(':empty') == ('Pseudo[Element[*]:empty]', None)
 
         # Special cases for CSS 2.1 pseudo-elements
-        assert parse_one(':before') == ('Element[*]', 'before')
-        assert parse_one(':after') == ('Element[*]', 'after')
-        assert parse_one(':first-line') == ('Element[*]', 'first-line')
-        assert parse_one(':first-letter') == ('Element[*]', 'first-letter')
+        assert parse_one(':BEfore') == ('Element[*]', 'before')
+        assert parse_one(':aftER') == ('Element[*]', 'after')
+        assert parse_one(':First-Line') == ('Element[*]', 'first-line')
+        assert parse_one(':First-Letter') == ('Element[*]', 'first-letter')
 
-        assert parse_one('::before') == ('Element[*]', 'before')
-        assert parse_one('::after') == ('Element[*]', 'after')
-        assert parse_one('::first-line') == ('Element[*]', 'first-line')
-        assert parse_one('::first-letter') == ('Element[*]', 'first-letter')
+        assert parse_one('::befoRE') == ('Element[*]', 'before')
+        assert parse_one('::AFter') == ('Element[*]', 'after')
+        assert parse_one('::firsT-linE') == ('Element[*]', 'first-line')
+        assert parse_one('::firsT-letteR') == ('Element[*]', 'first-letter')
 
-        assert parse_one('::selection') == ('Element[*]', 'selection')
+        assert parse_one('::text-content') == ('Element[*]', 'text-content')
+        assert parse_one('::attr(name)') == (
+            "Element[*]", "FunctionalPseudoElement[::attr(['name'])]")
+
+        assert parse_one('::Selection') == ('Element[*]', 'selection')
         assert parse_one('foo:after') == ('Element[foo]', 'after')
         assert parse_one('foo::selection') == ('Element[foo]', 'selection')
         assert parse_one('lorem#ipsum ~ a#b.c[href]:empty::selection') == (
@@ -265,8 +271,6 @@ class TestCssselect(unittest.TestCase):
             "Expected ident or '*', got <DELIM '#' at 1>")
         assert get_error('[foo=#]') == (
             "Expected string or ident, got <DELIM '#' at 5>")
-        assert get_error(':nth-child()') == (
-            "Expected at least one argument, got <DELIM ')' at 11>")
         assert get_error('[href]a') == (
             "Expected selector, got <IDENT 'a' at 6>")
         assert get_error('[rel=stylesheet]') == None
@@ -294,7 +298,7 @@ class TestCssselect(unittest.TestCase):
 
     def test_translation(self):
         def xpath(css):
-            return str(GenericTranslator().css_to_xpath(css, prefix=''))
+            return _unicode(GenericTranslator().css_to_xpath(css, prefix=''))
 
         assert xpath('*') == "*"
         assert xpath('e') == "e"
@@ -345,15 +349,17 @@ class TestCssselect(unittest.TestCase):
         assert xpath('e:only-of-type') == (
             "e[last() = 1]")
         assert xpath('e:empty') == (
-            "e[not(*) and not(normalize-space())]")
+            "e[not(*) and not(string-length())]")
+        assert xpath('e:EmPTY') == (
+            "e[not(*) and not(string-length())]")
         assert xpath('e:root') == (
             "e[not(parent::*)]")
         assert xpath('e:hover') == (
             "e[0]")  # never matches
         assert xpath('e:contains("foo")') == (
-            "e[contains(string(.), 'foo')]")
-        assert xpath('e:contains(foo)') == (
-            "e[contains(string(.), 'foo')]")
+            "e[contains(., 'foo')]")
+        assert xpath('e:ConTains(foo)') == (
+            "e[contains(., 'foo')]")
         assert xpath('e.warning') == (
             "e[@class and contains("
                "concat(' ', normalize-space(@class), ' '), ' warning ')]")
@@ -361,7 +367,7 @@ class TestCssselect(unittest.TestCase):
             "e[@id = 'myid']")
         assert xpath('e:not(:nth-child(odd))') == (
             "e[not((position() -1) mod 2 = 0 and position() >= 1)]")
-        assert xpath('e:not(*)') == (
+        assert xpath('e:nOT(*)') == (
             "e[0]")  # never matches
         assert xpath('e f') == (
             "e/descendant-or-self::*/f")
@@ -376,14 +382,15 @@ class TestCssselect(unittest.TestCase):
 
         # Invalid characters in XPath element names
         assert xpath(r'di\a0 v') == (
-            "*[name() = 'di\xa0v']")
+            u("*[name() = 'di v']"))  # di\xa0v
         assert xpath(r'di\[v') == (
             "*[name() = 'di[v']")
         assert xpath(r'[h\a0 ref]') == (
-            "*[attribute::*[name() = 'h\xa0ref']]")
+            u("*[attribute::*[name() = 'h ref']]"))  # h\xa0ref
         assert xpath(r'[h\]ref]') == (
             "*[attribute::*[name() = 'h]ref']]")
 
+        self.assertRaises(ExpressionError, xpath, u(':fİrst-child'))
         self.assertRaises(ExpressionError, xpath, ':first-of-type')
         self.assertRaises(ExpressionError, xpath, ':only-of-type')
         self.assertRaises(ExpressionError, xpath, ':last-of-type')
@@ -434,6 +441,71 @@ class TestCssselect(unittest.TestCase):
         assert css_to_xpath('*[aval="\'\\20\r\n \'"]') == (
             '''descendant-or-self::*[@aval = "'  '"]''')
 
+    def test_xpath_pseudo_elements(self):
+        class CustomTranslator(GenericTranslator):
+            def xpath_pseudo_element(self, xpath, pseudo_element):
+                if isinstance(pseudo_element, FunctionalPseudoElement):
+                    method = 'xpath_%s_functional_pseudo_element' % (
+                        pseudo_element.name.replace('-', '_'))
+                    method = _unicode_safe_getattr(self, method, None)
+                    if not method:
+                        raise ExpressionError(
+                            "The functional pseudo-element ::%s() is unknown"
+                        % pseudo_element.name)
+                    xpath = method(xpath, pseudo_element.arguments)
+                else:
+                    method = 'xpath_%s_simple_pseudo_element' % (
+                        pseudo_element.replace('-', '_'))
+                    method = _unicode_safe_getattr(self, method, None)
+                    if not method:
+                        raise ExpressionError(
+                            "The pseudo-element ::%s is unknown"
+                            % pseudo_element)
+                    xpath = method(xpath)
+                return xpath
+
+            # functional pseudo-class:
+            # elements that have a certain number of attributes
+            def xpath_nb_attr_function(self, xpath, function):
+                nb_attributes = int(function.arguments[0].value)
+                return xpath.add_condition(
+                    "count(@*)=%d" % nb_attributes)
+
+            # pseudo-class:
+            # elements that have 5 attributes
+            def xpath_five_attributes_pseudo(self, xpath):
+                return xpath.add_condition("count(@*)=5")
+
+            # functional pseudo-element:
+            # element's attribute by name
+            def xpath_attr_functional_pseudo_element(self, xpath, arguments):
+                attribute_name = arguments[0].value
+                other = XPathExpr('@%s' % attribute_name, '', )
+                return xpath.join('/', other)
+
+            # pseudo-element:
+            # element's text() nodes
+            def xpath_text_node_simple_pseudo_element(self, xpath):
+                other = XPathExpr('text()', '', )
+                return xpath.join('/', other)
+
+            # pseudo-element:
+            # element's href attribute
+            def xpath_attr_href_simple_pseudo_element(self, xpath):
+                other = XPathExpr('@href', '', )
+                return xpath.join('/', other)
+
+        def xpath(css):
+            return _unicode(CustomTranslator().css_to_xpath(css))
+
+        assert xpath(':five-attributes') == "descendant-or-self::*[count(@*)=5]"
+        assert xpath(':nb-attr(3)') == "descendant-or-self::*[count(@*)=3]"
+        assert xpath('::attr(href)') == "descendant-or-self::*/@href"
+        assert xpath('::text-node') == "descendant-or-self::*/text()"
+        assert xpath('::attr-href') == "descendant-or-self::*/@href"
+        assert xpath('p img::attr(src)') == (
+            "descendant-or-self::p/descendant-or-self::*/img/@src")
+
     def test_series(self):
         def series(css):
             selector, = parse(':nth-child(%s)' % css)
@@ -462,6 +534,30 @@ class TestCssselect(unittest.TestCase):
         assert series('foo') == None
         assert series('n+') == None
 
+    def test_lang(self):
+        document = etree.fromstring(XMLLANG_IDS)
+        sort_key = dict(
+            (el, count) for count, el in enumerate(document.getiterator())
+        ).__getitem__
+        css_to_xpath = GenericTranslator().css_to_xpath
+
+        def langid(selector):
+            xpath = css_to_xpath(selector)
+            items = document.xpath(xpath)
+            items.sort(key=sort_key)
+            return [element.get('id', 'nil') for element in items]
+
+        assert langid(':lang("EN")') == ['first', 'second', 'third', 'fourth']
+        assert langid(':lang("en-us")') == ['second', 'fourth']
+        assert langid(':lang(en-nz)') == ['third']
+        assert langid(':lang(fr)') == ['fifth']
+        assert langid(':lang(ru)') == ['sixth']
+        assert langid(":lang('ZH')") == ['eighth']
+        assert langid(':lang(de) :lang(zh)') == ['eighth']
+        assert langid(':lang(en), :lang(zh)') == [
+            'first', 'second', 'third', 'fourth', 'eighth']
+        assert langid(':lang(es)') == []
+
     def test_select(self):
         document = etree.fromstring(HTML_IDS)
         sort_key = dict(
@@ -549,7 +645,7 @@ class TestCssselect(unittest.TestCase):
         assert pcss('p:only-of-type') == ['paragraph']
         assert pcss('a:empty', 'a:EMpty') == ['name-anchor']
         assert pcss('li:empty') == [
-            'third-li', 'fourth-li', 'fifth-li', 'sixth-li', 'seventh-li']
+            'third-li', 'fourth-li', 'fifth-li', 'sixth-li']
         assert pcss(':root', 'html:root') == ['html']
         assert pcss('li:root', '* :root') == []
         assert pcss('*:contains("link")', ':CONtains("link")') == [
@@ -673,6 +769,20 @@ class TestCssselect(unittest.TestCase):
         assert count('div[class!=madeup]') == 243 # ? Seems right
         assert count('div[class~=dialog]') == 51 # ? Seems right
 
+XMLLANG_IDS = '''
+<test>
+  <a id="first" xml:lang="en">a</a>
+  <b id="second" xml:lang="en-US">b</b>
+  <c id="third" xml:lang="en-Nz">c</c>
+  <d id="fourth" xml:lang="En-us">d</d>
+  <e id="fifth" xml:lang="fr">e</e>
+  <f id="sixth" xml:lang="ru">f</f>
+  <g id="seventh" xml:lang="de">
+    <h id="eighth" xml:lang="zh"/>
+  </g>
+</test>
+'''
+
 HTML_IDS = '''
 <html id="html"><head>
   <link id="link-href" href="foo" />
