@@ -26,6 +26,12 @@ else:
     _unicode = str
 
 
+def _unicode_safe_getattr(obj, name, default=None):
+    # getattr() with a non-ASCII name fails on Python 2.x
+    name = name.encode('ascii', 'replace').decode('ascii')
+    return getattr(obj, name, default)
+
+
 class ExpressionError(SelectorError, RuntimeError):
     """Unknown or unsupported selector (eg. pseudo-class)."""
 
@@ -102,6 +108,19 @@ class GenericTranslator(object):
     of element names and attribute names.
 
     """
+    
+    ####
+    ####  HERE BE DRAGONS
+    ####
+    ####  You are welcome to hook into this to change some behavior,
+    ####  but do so at your own risks.
+    ####  Until is has recieved a lot more work and review,
+    ####  I reserve the right to change this API in backward-incompatible ways
+    ####  with any minor version of cssselect.
+    ####  See https://github.com/SimonSapin/cssselect/pull/22
+    ####  -- Simon Sapin.
+    ####
+
     combinator_mapping = {
         ' ': 'descendant',
         '>': 'child',
@@ -146,6 +165,9 @@ class GenericTranslator(object):
     lower_case_attribute_names = False
     lower_case_attribute_values = False
 
+    # class used to represent and xpath expression
+    xpathexpr_cls = XPathExpr
+
     def css_to_xpath(self, css, prefix='descendant-or-self::'):
         """Translate a *group of selectors* to XPath.
 
@@ -162,14 +184,9 @@ class GenericTranslator(object):
             The equivalent XPath 1.0 expression as an Unicode string.
 
         """
-        selectors = parse(css)
-        for selector in selectors:
-            if selector.pseudo_element:
-                raise ExpressionError('Pseudo-elements are not supported.')
-
         return ' | '.join(
             self.selector_to_xpath(selector, prefix)
-            for selector in selectors)
+            for selector in parse(css))
 
     def selector_to_xpath(self, selector, prefix='descendant-or-self::'):
         """Translate a parsed selector to XPath.
@@ -190,9 +207,20 @@ class GenericTranslator(object):
         if not tree:
             raise TypeError('Expected a parsed selector, got %r' % (selector,))
         xpath = self.xpath(tree)
-        assert isinstance(xpath, XPathExpr)  # help debug a missing 'return'
+        assert isinstance(xpath, self.xpathexpr_cls)  # help debug a missing 'return'
+        if selector.pseudo_element:
+            xpath = self.xpath_pseudo_element(xpath, selector.pseudo_element)
         return (prefix or '') + _unicode(xpath)
 
+    def xpath_pseudo_element(self, xpath, pseudo_element):
+        """Translate a pseudo-element.
+
+        Defaults to not supporting pseudo-elements at all,
+        but can be overridden by sub-classes.
+
+        """
+        raise ExpressionError('Pseudo-elements are not supported.')
+
     @staticmethod
     def xpath_literal(s):
         s = _unicode(s)
@@ -210,7 +238,9 @@ class GenericTranslator(object):
     def xpath(self, parsed_selector):
         """Translate any parsed selector object."""
         type_name = type(parsed_selector).__name__
-        method = getattr(self, 'xpath_%s' % type_name.lower())
+        method = getattr(self, 'xpath_%s' % type_name.lower(), None)
+        if method is None:
+            raise ExpressionError('%s is not supported.' %  type_name)
         return method(parsed_selector)
 
 
@@ -235,7 +265,7 @@ class GenericTranslator(object):
     def xpath_function(self, function):
         """Translate a functional pseudo-class."""
         method = 'xpath_%s_function' % function.name.replace('-', '_')
-        method = getattr(self, method, None)
+        method = _unicode_safe_getattr(self, method, None)
         if not method:
             raise ExpressionError(
                 "The pseudo-class :%s() is unknown" % function.name)
@@ -244,7 +274,7 @@ class GenericTranslator(object):
     def xpath_pseudo(self, pseudo):
         """Translate a pseudo-class."""
         method = 'xpath_%s_pseudo' % pseudo.ident.replace('-', '_')
-        method = getattr(self, method, None)
+        method = _unicode_safe_getattr(self, method, None)
         if not method:
             # TODO: better error message for pseudo-elements?
             raise ExpressionError(
@@ -301,7 +331,7 @@ class GenericTranslator(object):
             # http://www.w3.org/TR/css3-namespace/#prefixes
             element = '%s:%s' % (selector.namespace, element)
             safe = safe and is_safe_name(selector.namespace)
-        xpath = XPathExpr(element=element)
+        xpath = self.xpathexpr_cls(element=element)
         if not safe:
             xpath.add_name_test()
         return xpath
@@ -397,7 +427,7 @@ class GenericTranslator(object):
                 % function.arguments)
         value = function.arguments[0].value
         return xpath.add_condition(
-            'contains(string(.), %s)' % self.xpath_literal(value))
+            'contains(., %s)' % self.xpath_literal(value))
 
     def xpath_lang_function(self, xpath, function):
         if function.argument_types() not in (['STRING'], ['IDENT']):
@@ -406,12 +436,7 @@ class GenericTranslator(object):
                 % function.arguments)
         value = function.arguments[0].value
         return xpath.add_condition(
-            "ancestor-or-self::*[@lang][1][starts-with(concat("
-                # XPath 1.0 has no lower-case function...
-                "translate(@%s, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', "
-                               "'abcdefghijklmnopqrstuvwxyz'), "
-                "'-'), %s)]"
-            % (self.lang_attribute, self.xpath_literal(value.lower() + '-')))
+            "lang(%s)" % (self.xpath_literal(value)))
 
 
     # Pseudo: dispatch by pseudo-class name
@@ -455,7 +480,7 @@ class GenericTranslator(object):
         return xpath.add_condition('last() = 1')
 
     def xpath_empty_pseudo(self, xpath):
-        return xpath.add_condition("not(*) and not(normalize-space())")
+        return xpath.add_condition("not(*) and not(string-length())")
 
     def pseudo_never_matches(self, xpath):
         """Common implementation for pseudo-classes that never match."""
@@ -571,6 +596,20 @@ class HTMLTranslator(GenericTranslator):
                 "and (name(.) = 'input' or name(.) = 'command')"
                 "and (@type = 'checkbox' or @type = 'radio'))")
 
+    def xpath_lang_function(self, xpath, function):
+        if function.argument_types() not in (['STRING'], ['IDENT']):
+            raise ExpressionError(
+                "Expected a single string or ident for :lang(), got %r"
+                % function.arguments)
+        value = function.arguments[0].value
+        return xpath.add_condition(
+            "ancestor-or-self::*[@lang][1][starts-with(concat("
+                # XPath 1.0 has no lower-case function...
+                "translate(@%s, 'ABCDEFGHIJKLMNOPQRSTUVWXYZ', "
+                               "'abcdefghijklmnopqrstuvwxyz'), "
+                "'-'), %s)]"
+            % (self.lang_attribute, self.xpath_literal(value.lower() + '-')))
+
     def xpath_link_pseudo(self, xpath):
         return xpath.add_condition("@href and "
             "(name(.) = 'a' or name(.) = 'link' or name(.) = 'area')")
